/** This file replicates Table 2 and Figure 6 in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression".
	It requires CVtables (produced by Critical_Value_Tables.ox)
**/


#include <oxstd.oxh>
#import <database>
#import <modelbase>
#include <oxdraw.h>
#include <interpolate.ox>		

GetEigVals(const mytil, const mx, const mztil) 
/** Computes the max and min eigenvalues needed to perform the conditional subvector AR test
	@param mytil [n][p] matrix of endogenous variables
	@param mx [n][q] matrix of included exogenous variables (could be empty)
	@param mztil [n][k] matrix of excluded exogenous varialbes (instruments)
	@return vector[2] of the min and max eigenvalues of (2.14) in GKM2
**/
{
	decl arg, mz = mztil, my = mytil;
	if(mx != <>){
		olsc(mztil, mx, &arg);
		mz = mztil-mx*arg;
		olsc(mytil, mx, &arg);
		my = mytil-mx*arg;
	}
	olsc(my, mz, &arg);
	decl mv = my-mz*arg;

	decl p = columns(mv);
	decl k = columns(mz);
	decl n = sizeof(mv);
	mz *= choleski(invertgen(mz'mz/n));					// standardize to impose rotational invariance
	decl df = n - k - columns(mx);						// small sample degrees of freedom for estimation of variance
	decl mvvinvch = choleski(invertgen(mv'mv/df));
	mv *= mvvinvch;										// standardize to impose rotational invariance
	my *= mvvinvch;										// perform the same standardization on y
	decl mA, meig;										// matrix whose eigenvalues are used in cond sub AR test
	mA = my'mz*mz'my/n;
	eigensym(mA, &meig);
	return min(meig)|max(meig);

}
condARtest(const x, const overid, const dlevel)
/**	Performs the conditional subvector AR test of GKM
	@param x: vector[2] smallest~largest eigenvalues
	@param overid: integer, number of overidentifying restrictions
	@param dlevel: double, significance level
	@return: binary, test outcome, TRUE= reject H0
**/
{
	decl quants = loadmat(sprint("CVtables\\k_",overid+1,"_level_",1-dlevel,".in7"));   // array holding matrix of conditional quantile bins for the conditional test (x ~ cv)
	decl idx = sumc(quants[][0].<x[1]);													// index of ceiling in x1 values (to be used as conditional cv)
	decl x0, x_lo, x_up;
	x0 = x[1];
	x_lo = quants[idx-1][0];
	x_up = quants[idx][0];
	decl y_up =  quants[idx][1], y_lo = quants[idx-1][1];
	decl dcv = interpolate(x0, x_up, x_lo, y_up, y_lo);			    // conditional critical value interpolated from table
	return x[0] .> dcv;												// conditional subAR test function
}

condARtestStat(const x, const overid, const dlevel)
/**	Performs the conditional subvector AR test of GKM
	@param x: vector[2] smallest~largest eigenvalues
	@param overid: integer, number of overidentifying restrictions
	@param dlevel: double, significance level
	@return: binary, test outcome, TRUE= reject H0
**/
{
	decl quants = loadmat(sprint("CVtables\\k_",overid+1,"_level_",1-dlevel,".in7"));   // array holding matrix of conditional quantile bins for the conditional test (x ~ cv)
	decl idx = sumc(quants[][0].<x[1]);													// index of ceiling in x1 values (to be used as conditional cv)
	decl x0, x_lo, x_up;
	x0 = x[1];
	x_lo = quants[idx-1][0];
	x_up = quants[idx][0];
	decl y_up =  quants[idx][1], y_lo = quants[idx-1][1];
	decl dcv = interpolate(x0, x_up, x_lo, y_up, y_lo);			    // conditional critical value interpolated from table
	return (x[0] ~ dcv);											// conditional subAR test function
}

subARtestSet(const mY, const mZ, const mX, const cl, const cu, const dstep, const alpha, const fPlot)
{
	// Test of rank condition on mY[][1] - if non-reject, beta is unidentified.
	decl k = columns(mZ);
	decl p = columns(mY);
	decl n = rows(mY);
	decl mYW, evals;

	decl mF = .NaN, mSubAR_ID, dcond = .NaN, asCIs = new array[2];
		
	// Identification test
	decl iXinY = vecindex(meanc(mZ .== mY[][1]) .== 1);		// compute index of mY[][1] in mZ
	decl fXinY = !(iXinY == <>);							// TRUE if exogenous regressor in position Y[][1]
	mYW = mY[][1+fXinY:];									// consider only the endogenous regressors
	evals = GetEigVals(mYW, mX ~ (fXinY ? mY[][1] : <>), fXinY ? dropc(mZ, iXinY) : mZ);
	if(p-fXinY == 2){ 											// if there's only one, do a standard AR test = first-stage F
//		println("SubAR (ID) statistic: ", evals[0], " and p-value ", tailchi(evals[0], k-fXinY));
		mF = evals[0]/(k-fXinY) | tailchi(evals[0],k-fXinY);		// first-stage F and asy p-value
		mSubAR_ID = evals[0] | tailchi(evals[0],k-fXinY);			// subAR test and asy p-value
	}
	else{														// otherwise do subAR test
//		println("SubAR (ID) statistic: ", evals[0], ". Conditioning statistic: ", evals[1]);
//		println("conditional subAR confidence set is ", condARtest(evals, k-p+2/*Df are (k - mw)*/, alpha) ? "bounded" : "unbounded", " at alpha = ", alpha);
//		println("Unconditional p-value: ", 1-probchi(evals[0], k-p+2));
		mSubAR_ID = evals[0] | tailchi(evals[0],k-p+2);		// subAR test and asy p-value
		dcond = evals[1];									// conditioning statistic
	}

	
	// Create matrix of hypothesised values with column for storing test result	
	decl mH0 = range(cl, cu, dstep)' ;	// Create vector of hypothesised coefficient values
	decl csteps = sizeof(mH0) ;				// number of steps
	mH0 ~= zeros(csteps, 9) ;				// Append rx4 matrix of zeros to mH0 to store results from subAR test
	
	 // Runs subAR test for each hypothesised value in mH0
	for(decl i=0; i<csteps; ++i) {
		if(p==2){
			mYW = (mY[][0] - mH0[i][0]*mY[][1]);  				// Impose null hypothesis
			evals = GetEigVals(mYW, mX, mZ);
			mH0[i][1] = mH0[i][2] = 1-probchi(evals[0], k-p+2) > alpha ;		// cond = uncond AR, since no endogenous var to concentrate out
		}
		else{
			mYW = (mY[][0] - mH0[i][0]*mY[][1]) ~ mY[][2:] ;  		// Impose null hypothesis
			evals = GetEigVals(mYW, mX, mZ) ;		  				// Run test on null hypothesis			
			mH0[i][1] = 1-condARtest(evals, k-p+2, alpha) ; 		// Store test result, with 1 indicating a failure to reject H0
			mH0[i][2] = 1-probchi(evals[0], k-p+2) > alpha ;		// Store results comparing test stat against chi-squared dist; 1 indicates failure to reject H0
		}
		if(fPlot){
			mH0[i][3:4] = condARtestStat(evals, k-p+2, 0.01);		// Add conditional statistic and critical values, 1 percent
			mH0[i][5] = quanchi(0.99, k-p+2) ;						// Add unconditional critical values, 1 percent
			mH0[i][6] = condARtestStat(evals, k-p+2, 0.05)[1];		// Add conditional critical values, 5 percent
			mH0[i][7] = quanchi(0.95, k-p+2) ;						// Add unconditional critical values, 5 percent
			mH0[i][8] = condARtestStat(evals, k-p+2, 0.1)[1];		// Add conditional critical values, 10 percent
			mH0[i][9] = quanchi(0.9, k-p+2) ;						// Add unconditional critical values, 10 percent
		}
	}
	decl mD = diff(0 | mH0[][1] | 0)~diff(0 | mH0[][2] | 0) ;									  // Pad vector with zeros so that all 1's and -1's respectively correspond to start and end of an interval. 
	decl mStart, mEnd;
	for(decl l=0; l<2; ++l) {
		mStart = vecrindex(mD[1:csteps][l].==1) ;	// Get index of starting values
		mEnd = vecrindex(mD[2:csteps+1][l].==-1) ;	// Get index of ending values
		asCIs[l] = sprint("$[","%.3f", mH0[mStart][0],", ","%.3f", mH0[mEnd][0],"]$");		// return the confidence interval 
		if(fPlot){
			DrawLine(0, mH0[mStart][0], 0, mH0[mStart][0], max(mH0[][3:5]), l ? 6 : 8);
			DrawLine(0, mH0[mEnd][0], 0, mH0[mEnd][0], max(mH0[][3:5]), l ? 6 : 8);
		}
	}

	if(fPlot){
		decl asnames = {"SubAR statistic", "Conditional cv", "Unconditional cv"};
		DrawXMatrix(0, mH0[][3:9]', asnames, mH0[][0]', "", 0, <2;8;6;8;6;8;6>);
		DrawText(0, "$\\beta_0$", -1, -1, -1, -1, TEXT_XLABEL);  
		DrawText(0, "$\\alpha=1\%$", cl, 1.01*mH0[0][5], -1, -1);  
		DrawText(0, "$\\alpha=5\%$", cl, 1.02*mH0[0][7], -1, -1);  
		DrawText(0, "$\\alpha=10\%$", cl, 1.03*mH0[0][9], -1, -1);
	
		DrawAdjust(0, ADJ_LEGEND, /*# of columns*/1);
		ShowDrawWindow();
	}
	return {asCIs, mF, mSubAR_ID, dcond};
}
TexTable(const f, const asnames, const mestimates, const asCIs, const mF, const mSubAR, const dcond)
/**	returns Tabular environment in tex
	@param f, string, file name, extension must be .tex
	@param asnames, array[p] variable names in model
	@param mestimates, [p][2] 2SLS estimates of coefficients on asnames in each specification
	@param asCIs, array[p] of arrays[2*3] of Wald, conditional and unconditional AR CIs
	@param mF, [2][1] First-stage F stat and p value for spec 1
	@param mSubAR, [2][2] Sub-AR ID stat and p value for specs 1 and 2
	@param dcond, double, conditioning statistic for specification 2
**/
{
	//f: filename
	//asnames: variable names (row headings)
	//mstuff: data
	decl cp = sizeof(asnames);
	fprintln(f,"\\begin{tabular}{lrrrr}");
	fprintln(f,"\\hline \\hline");
	fprintln(f," & \\multicolumn{4}{c}{Dependent variable}\\\\");
	fprintln(f,"\\cline{2-5}");
	fprintln(f," & \\multicolumn{4}{c}{$\\sigma$ (value function curvature)}\\\\");
	fprintln(f,"\\cline{2-5}");
	fprintln(f," & \\multicolumn{2}{c}{\\underline{Specification 1}} & \\multicolumn{2}{c}{\\underline{Specification 2}}\\\\");
	for(decl j = 0; j < cp; ++j)
	{
		fprint(f, asnames[j], " & ");
		fprint(f,"%.3f", isnan(mestimates[j][0]) ? "" : mestimates[j][0], " & ");			// point estimate for coefficient on variable j in specification 1
		fprint(f,(asCIs[j][0]=="") ? "" : "W: ", asCIs[j][0], " & ");					// Wald CI for coefficient on variable j in specification 1
		fprint(f,"%.3f", isnan(mestimates[j][1]) ? "" : mestimates[j][1], " & ");			// point estimate for coefficient on variable j in specification 2
		fprintln(f, asCIs[j][3], "\\\\ ");						// Wald CI for coefficient on variable j in specification 2
		fprintln(f," & & ", (asCIs[j][1]=="") ? "" : "C: ", asCIs[j][1], " & & ",asCIs[j][4], "\\\\ ");					// conditional CI for coefficient on variable j in each spec
		fprintln(f," & & ", (asCIs[j][2]=="") ? "" : "U: ", asCIs[j][2], " & & ",asCIs[j][5], "\\\\ ");					// uconditional CI for coefficient on variable j in each spec
	}
	fprintln(f,"First-stage $F$ statistic & ","%.3f", mF[0], " & $p$-value ","%.3f", mF[1], " & & \\\\");			// first-stage F stat in spec 1 
	fprintln(f,"Sub. AR statistic (ID) & ","%.3f", mSubAR[0][0], " & $p$-value ","%.3f", mSubAR[1][0], " & ","%.3f", mSubAR[0][1], " & $p$-value ","%.3f", mSubAR[1][1], " \\\\");			// sub-AR Identification stat and pvalue
	fprintln(f,"Conditioning statistic & $\\infty$ & & ","%.3f", dcond, " & \\\\"); 
	fprintln(f,"\\hline \\hline \\end{tabular}");
}


main()
{
	decl time = timer();
	decl data = new Database();							// create database object (to load data)
	data.LoadIn7("20060431_risk.in7");					// load data
	data.Deterministic(FALSE);							// create constant in the database
	decl iY = 0, iX = 1, iZ = 2;						// give names to the three different groups of variables, Y endogenous, X controls, Z instruments

	decl alpha = 0.05; 									// significance level

	decl asXvars = {"chinese", "age", "gender", "edu", "market", "south", "Constant"};
	decl asnames = asXvars~{"income (IV)", "Relative Income (IV)", "Mean income (IV)"}; 		// all variable names in both specs: 1 uses income, 2 uses relative and mean income
	decl mestimates = constant(.NaN,sizeof(asnames),2);	// to store point estimates in each specification
	decl asCIs = new array[sizeof(asnames)];
	for(decl i = 0; i < sizeof(asnames); ++i)
		asCIs[i] = {"","","","","",""};					// store 3 CIs per parameter and spec, initialize empty
	decl mF = new matrix[2][1];							// first-stage F for spec 1
	decl mSubAR = new matrix[2][2];						// subAR ID test for each spec
	decl dcond;											// conditioning statistic in spec 2

	decl vlo = <-1;-.1;-.6;-.1;-.1;-1;0;-.1;-.5;-.02>;	// lower bounds for CIs (from prior estimation)
	decl vup = < 1; .1; .8; .1; .1;.1;2; .1; 1; .04>;
	decl dstep = 0.001;
	for(decl ispec = 0; ispec < 2; ++ispec){			// loop over the two different specifications: 0 is with income, 1 is with mnincome and relincome
	
		println("\nSpecification ", ispec+1);
		for(decl j = 0; j < sizeof(asXvars); ++j){
			data.DeSelect();
			if(ispec == 0)
				data.Select(iY, {"vfctnc", 0, 0, asXvars[j], 0, 0, "income", 0, 0});  // select endogenous variables Y: syntax is for time series: {"varname", lag1, lag2, "varname", lag1, lag2, etc}. For cross-section, set lag1=lag2=0.
			else
				data.Select(iY, {"vfctnc", 0, 0, asXvars[j], 0, 0, "mnincome", 0, 0, "nmlrlincome", 0, 0});  // two endogenous RHS variables in second specification

//			println(asXvars[j]);
			for(decl i = 0; i < sizeof(asXvars); ++i){
				if(i == j) continue;
				data.Select(iX, {asXvars[i], 0, 0});							  				// select exogenous variables X, except the jth
			}
			data.Select(iZ, {"rainfall", 0, 0, "headnowork", 0, 0, asXvars[j], 0, 0});			// select exogenous variables Z
			data.SetSelSample(/*iYear1*/-1, /*iPeriod1*/1, /*iYear2*/-1, /*iPeriods2*/1);		// set estimation sample, use -1 for smallest/largest values
	
			decl mY = data.GetGroup(iY);						// obtain Y
			decl mX = data.GetGroup(iX);						// obtain X
			decl mZ = data.GetGroup(iZ);						// obtain Z
		
			decl asCI, mFdum, mSub;
			if(ispec == 0 && asXvars[j] == "gender"){
				SetDrawWindow("gender");
				[asCI, mFdum, mSub, dcond] = subARtestSet(mY, mZ, mX, /*lower*/vlo[j], /*upper*/vup[j], /*step size*/dstep, alpha, /*fPlot*/TRUE);		// compute AR confidence sets
			}
			else
				[asCI, mFdum, mSub, dcond] = subARtestSet(mY, mZ, mX, /*lower*/vlo[j], /*upper*/vup[j], /*step size*/dstep, alpha, /*fPlot*/FALSE);	// do not plot except for gender
			asCIs[j][<1:2>+ispec*3] = asCI;
		}
//		println("Endogenous regressor(s)");
		if(ispec == 0){
//			println("Income");
			data.DeSelect();
			data.Select(iY, {"vfctnc", 0, 0, "income", 0, 0});  // select endogenous variables Y: syntax is for time series: {"varname", lag1, lag2, "varname", lag1, lag2, etc}. For cross-section, set lag1=lag2=0.
	 		for(decl i = 0; i < sizeof(asXvars); ++i)
				data.Select(iX, {asXvars[i], 0, 0});							  // select exogenous variables X, except the jth
			data.Select(iZ, {"rainfall", 0, 0, "headnowork", 0, 0});				// select exogenous variables Z
			data.SetSelSample(/*iYear1*/-1, /*iPeriod1*/1, /*iYear2*/-1, /*iPeriods2*/1);					// set estimation sample, use -1 for smallest/largest values
	
			decl mY = data.GetGroup(iY);						// obtain Y
			decl mX = data.GetGroup(iX);						// obtain X
			decl mZ = data.GetGroup(iZ);						// obtain Z

			// Standard parametric confidence interval for 2sls
			decl ca = quann(1-alpha/2);
			decl mPi, mb, xxinv;
			olsc((mX~mY[][1:]), (mX~mZ), &mPi) ;
			decl mXh = (mX~mZ)*mPi ;   		 // Get predicted values from first stage
			olsc(mY[][0], mXh, &mb, &xxinv) ;	  	 // Get second stage coefficients
			decl mVar, mV;
			mV = (mY[][0] - (mX~mY[][1:])*mb);
			mVar = mV'mV*xxinv/(sizeof(mY) - columns(mZ));												  // Calculate variance-covariance matrix
			decl mSE = sqrt(diagonal(mVar))' ;										  // Get standard errors
			decl mCI = mb - ca*mSE ~ mb + ca*mSE ; 									  // Construct confidence interval
			for(decl j = 0; j < columns(mXh); ++j)
				asCIs[j][0] = sprint("$[","%.3f", mCI[j][0],", ", "%.3f", mCI[j][1],"]$");		// return the confidence interval rounded to dstep decimals
			decl vidx = range(0, columns(mX));
			mestimates[vidx][0] = mb;				// store 2SLS estimates

			decl asCI, mSub;
			[asCI, mF, mSub, dcond] = subARtestSet(mY, mZ, mX, /*lower*/vlo[sizeof(asXvars)], /*upper*/vup[sizeof(asXvars)], /*step size*/dstep, alpha, /*fPlot*/FALSE);	// do not plot except for gender
			asCIs[sizeof(asXvars)][<1:2>] = asCI;
			mSubAR[][0] = mSub;
		}
		else{
			decl asYvars = {"nmlrlincome","mnincome"};
			for(decl j = 0; j < 2; ++j){
//				println(asYvars[j]);
				data.DeSelect();
				data.Select(iY, {"vfctnc", 0, 0, asYvars[j], 0, 0, asYvars[1-j], 0, 0});  // middle endogenous variable is tested
		 		for(decl i = 0; i < sizeof(asXvars); ++i)
					data.Select(iX, {asXvars[i], 0, 0});							  // select exogenous variables X, except the jth
				data.Select(iZ, {"rainfall", 0, 0, "headnowork", 0, 0});				// select exogenous variables Z
				data.SetSelSample(/*iYear1*/-1, /*iPeriod1*/1, /*iYear2*/-1, /*iPeriods2*/1);					// set estimation sample, use -1 for smallest/largest values
		
				decl mY = data.GetGroup(iY);						// obtain Y
				decl mX = data.GetGroup(iX);						// obtain X
				decl cX = columns(mX);
				decl mZ = data.GetGroup(iZ);						// obtain Z
				if(j == 0){				// for 2SLS, suffices to do it only once
					// Standard parametric confidence interval for 2sls
					decl ca = quann(1-alpha/2);
					decl mPi, mb, xxinv;
					olsc((mX~mY[][1:]), (mX~mZ), &mPi) ;
					decl mXh = (mX~mZ)*mPi ;   		 // Get predicted values from first stage
					olsc(mY[][0], mXh, &mb, &xxinv) ;	  	 // Get second stage coefficients
					decl mVar, mV;
					mV = (mY[][0] - (mX~mY[][1:])*mb);
					mVar = mV'mV*xxinv/(sizeof(mY) - columns(mZ));												  // Calculate variance-covariance matrix
					decl mSE = sqrt(diagonal(mVar))' ;										  // Get standard errors
					decl mCI = mb - ca*mSE ~ mb + ca*mSE ; 									  // Construct confidence interval
					for(decl j = 0; j < cX; ++j)
						asCIs[j][3] = sprint("$[","%.3f", mCI[j][0],", ", "%.3f", mCI[j][1],"]$");		// return the confidence interval rounded to dstep decimals
					asCIs[cX+1][3] = sprint("$[","%.3f", mCI[cX][0],", ", "%.3f", mCI[cX][1],"]$");		
					asCIs[cX+2][3] = sprint("$[","%.3f", mCI[cX+1][0],", ", "%.3f", mCI[cX+1][1],"]$");		
					decl vidx = range(0, cX-1)~(cX+<1,2>);
					mestimates[vidx][1] = mb;						// store 2SLS estimates
				}				
				decl asCI, mFdum, mSub;
				if(asYvars[j] == "mnincome"){
					SetDrawWindow("meanincome"); 
					[asCI, mFdum, mSub, dcond] = subARtestSet(mY, mZ, mX, /*lower*/vlo[cX+1+j], /*upper*/vup[cX+1+j], /*step size*/dstep/10, alpha, /*fPlot*/TRUE);	// do not plot except for mnincome
				}
				else 
					[asCI, mFdum, mSub, dcond] = subARtestSet(mY, mZ, mX, /*lower*/vlo[cX+1+j], /*upper*/vup[cX+1+j], /*step size*/dstep, alpha, /*fPlot*/FALSE);	
				asCIs[cX+j+1][<4:5>] = asCI;
				mSubAR[][1] = mSub;
			}
		}
	}
	decl f = fopen("Table2.tex", "w"); 
	fprintln(f,"\\documentclass{article}");
	fprintln(f,"\\usepackage{geometry}[1in]");
	fprintln(f,"\\begin{document}");
	
	TexTable(f, asnames, mestimates, asCIs, mF, mSubAR, dcond);

	fprintln(f,"\\end{document}");
	fclose(f);

	println("\nlapsed time: ", timespan(time),"\n");

}
