/**	This file replicates Figure 5 in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression".
	It requires CVtables\k_5_level_0.95.in7 (produced by Critical_Value_Tables.ox)
	Takes about 50 minutes on a 2.2GHz 44-core multi-threaded Windows server
**/

#include <oxstd.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#import <maximize>
#import <database>
#include <interpolate.ox>		

AR(const vtheta, const mY, const mZ)
/**	Computes AR statistic	**/
{
	decl mg = (mY*(1|-vtheta)).*mZ;
	decl vghat = meanc(mg)';
	decl mV = variance(mg);
	return sizeof(mg)*vghat'invertsym(mV)*vghat;
}
AR2(const vtheta, const mY, const mZ)
/**	Computes AR statistic	**/
{
	decl mg = (mY*(1|-vtheta)).*mZ;
	decl vghat = meanc(mg)';
	decl mV = variance(mg);
	decl mZY1 = mZ.*mY[][1];
	decl mGamma1 = -(mZY1-meanc(mZY1))'mg/sizeof(mg);
	decl mD = -mZ'mY[][1]/sizeof(mg) - mGamma1*invertsym(mV)*vghat;
	decl meigval, meigvec;
	eigensym(mV, &meigval, &meigvec);
	decl mVminushalf = meigvec*diag(1 ./ sqrt(meigval))*meigvec';
	decl vgtilde = mVminushalf*vghat;
	decl arg = mVminushalf*mD;
	decl M = unit(columns(mZ)) - arg*invertsym(arg'arg)*arg'; 
	return sizeof(mg)*vgtilde'M*vgtilde;
}
gammahat(const beta0, const mY, const mZ)
/**	Computes restricted estimate of gamma
**/
{
	decl vY0 = (mY[][0]-mY[][2]*beta0);

	decl mPz, mYb, veigv, meig, ret, vbhat; 
	mYb = vY0~mY[][1];
	mPz = mZ*invertsym(mZ'mZ)*mZ';
	return invertsym(mY[][1]'mPz*mY[][1])*mY[][1]'mPz*vY0;
}
ICS(const vtheta, const mY, const mZ)
/**	Computes identification category selection test	**/
{
	decl mg = (mY*(1|-vtheta)).*mZ;
	decl mV = variance(mg);
	decl mG1 = mY[][1].*mZ;
	decl mG1norm = sqrt(sumsqrr(mG1));
	decl sig1hat = sqrt(variance(mG1norm));
	return sqrt(mY[][1]'mZ*invertsym(mV)*mZ'mY[][1])/sig1hat/sizeof(mY);
}

condARtest(const x, const quants)
{
	// x = largest~smallest eigenvalues
	decl idx = sumc(quants[][0].<x[0]);								// index of ceiling in x1 values (to be used as conditional cv)
	decl x0, x_lo, x_up;
	x0 = x[0];
	x_lo = quants[idx-1][0];
	x_up = quants[idx][0];
	decl y_up =  quants[idx][1], y_lo = quants[idx-1][1];
	decl dcv = interpolate(x0, x_up, x_lo, y_up, y_lo);			    // conditional critical value interpolated from table
	return x[1] .> dcv;												// conditional subAR test function
}

main()
{
	decl time = timer();
	decl n = 250;
	decl k = 2;						// must be even for pibeta below
	decl crep = 1e4;
	println("k = ", k, ", n = ", n, ", nrep=", crep);
	decl alpha = 0.05;				// overall significance level
	decl alpha1 = 0.005;			// step 1 significance level
	decl alpha2 = alpha-alpha1;		// step 2 significance level
	decl K_L	= 0.05;				// cut-off for selection of data-dependent significance level, as in Andrews (2017, p. 34)

	decl gamma = 0;					// true value of gamma
	decl beta = 0;					// true value of beta
	decl vgammarange = range(-20, 20, .1);	// range of values of gamma
	decl pibetanorm = 4;					// identification strength of beta
	decl vpibeta = pibetanorm * (ones(k/2,1)|-ones(k/2,1))/sqrt(k);
	
	decl mV = <1, 0.8, 0.8;
			   0.8, 1, 0.3;
			   0.8, 0.3, 1>;	// covariance matrix of epsilon
	decl mVch = choleski(mV);
	decl astests = {"$\\varphi_{ACZ}$", "$\\varphi_{GKMC}$", "$\\varphi_c$"};	
	// conditional critical values for the condsubAR test
	decl quants = loadmat(sprint("CVtables\\k_",k,"_level_",1-alpha,".in7"));   	   // array holding matrix of conditional quantile bins for the conditional test (x ~ cv)

	MaxControl(1000,0);
	MaxControlEps(1e-5, 1e-10);


	decl beta0 = 0;							// value of beta under H0
	decl vbeta1 = range(-2,2,.05);			// values of beta under H1

	for(decl id = 0; id < 3; ++id){		// repeat with different identification strength for gamma
		decl pigammanorm = id == 0 ? 1 : id==1 ? 2 : 4; 		
		decl vpigamma = pigammanorm * ones(k,1)/sqrt(k);
	
		decl dcv1 = quanchi(1-alpha1, k);		// cv for step 1 (AR test)
		decl dcv2 = quanchi(1-alpha, k-1);		// cv for subAR test
		decl cpower = sizerc(vbeta1);
		decl mARAR = ones(crep,cpower);				// AR/AR test function, initialize at reject because it takes only one acceptance to change
		decl msubAR = ones(crep,cpower);			// sub AR test function (GKMC)
		decl mcondAR = ones(crep,cpower);			// conditional sub AR test function
		decl mICS = new matrix[crep][cpower];		// holds the ICS
		parallel for(decl p = 0; p < cpower; ++p){
			beta = vbeta1[p];				// set true beta
			ranseed(-1);
			for(decl i = 0; i < crep; ++i){
				decl mZ = rann(n, k);					// instruments
				decl me = rann(n, 3) * mVch';			// error component
				decl vU = me[][0];			// U
				decl vV_W= me[][1];			// V_W
				decl vV_Y= me[][2];			// V_Y
			
				decl vW = mZ * vpigamma/sqrt(n) + vV_W;
				decl vY = mZ * vpibeta/sqrt(n) + vV_Y;
				decl vy = vW * gamma + vY * beta + vU;
		
				// compute subvector AR test
				decl mYb = (vy - vY*beta0)~vW;
				decl arg1 =  (1 ~ gamma ~ beta | 0 ~ 1 ~ 0 | 0 ~ 0 ~ 1);
				decl mSigV = arg1*mV*arg1';											// variance matrix of reduced form errors
				decl arg2 =  (1 ~ 0 ~ -beta0 | 0 ~ 1 ~ 0);
				decl mO = arg2*mSigV*arg2', mZZinv=invertsym(mZ'mZ/n);
				
				decl mX = mYb'mZ*mZZinv*mZ'mYb*invertsym(mO)/n;
				decl eigv;
				eigen(mX, &eigv);
				msubAR[i][p] = min(eigv) > dcv2;
				mcondAR[i][p] = condARtest(max(eigv)~min(eigv),quants);				// conditional subAR test function
		
				decl vgammacs = vgammarange;
				for(decl j = 0; j < sizerc(vgammarange); ++j)
					vgammacs[j] = AR(vgammarange[j]|beta0,vy~vW~vY,mZ) > dcv1;		// test each value of gamma in range
				vgammacs = deleteifc(vgammarange, vgammacs);						// keep only the values that are accepted
				vgammacs = gammahat(beta0,vy~vW~vY,mZ)~vgammacs;					// append estimate 
				for(decl j = 0; j < sizerc(vgammacs); ++j){
					decl dICS = ICS(vgammacs[j]|beta0, vy~vW~vY,mZ);
					mICS[i][p] = dICS;
					decl alpha2hat = dICS > K_L ? alpha : alpha2;
					if(AR2(vgammacs[j]|beta0,vy~vW~vY,mZ) <= quanchi(1-alpha2hat, k-1)){	// if the second step test accepts
						mARAR[i][p] = 0;														// record zero in the test function
						break;																	// and break loop
					}
				}
			}
		}
		println("average ICS", meanc(mICS.>K_L));
		DrawTitle(id, sprint("$\\gamma$ ", id == 0 ? "weakly" : id == 1 ? "moderately" : "strongly", " identified"));
		DrawXMatrix(id, meanc(mARAR) | meanc(msubAR) | meanc(mcondAR) | alpha, astests,  vbeta1, "", 0, <2;3;4;1>);
		DrawText(id, "$\\beta$", 0, 0, -1, -1, TEXT_XLABEL);
		DrawAdjust(ADJ_AREA_Y, id, 0, id == 0 ? 0.2 : id == 1 ? 0.6 : 1);
		DrawLegend(id, 500, 0, id);
		DrawAdjust(ADJ_LEGEND, id, 3);
	}
	ShowDrawWindow();
	SaveDrawWindow("PowerACZ.eps");
	println("\nlapsed time: ", timespan(time),"\n");
}
