/**	This file computes the power of the subvector conditional AR and GKMC tests.
	It requires tables of conditional critical values, which should have been stored in the format: CVtables\k_<k>_level_0.95.in7, <k> = 2,5,10,20.
	If not computed, these critical values can be obtained by running the file Critical_Value_Tables.ox
**/

#include <oxstd.oxh>
#include <oxprob.oxh>
#include <oxfloat.oxh>
#import <database>
#include <interpolate.ox>		

main()									
{
	
	decl alpha = 0.05;					// significance level of test
	decl vk = <2,5,10,20>;				// vector contain different # of instruments
	decl ck = sizerc(vk);
	//parameters for alternative
	decl crepalt = 1e5;	   				// # of MC repetitions under alternative. Keep same as in powerbounds.ox
	decl ckappa2 = 30; 					// # of kappa2 points
	decl kappa2_lb = 0.1;				// lower bound of kappa2, very close to null
	decl vkap1mkap2 = 0~(2 .^<0:6>);	// kappa1-kappa2 points under H1. 
	decl ckap1mkap2 = sizerc(vkap1mkap2);
	decl avkappa2 = new array[ck];
	for(decl i = 0; i < ck; ++i){
		decl up = i == 0 ? 25 : i == 1 ? 30 : i == 2 ? 38 : 46;	 		// different range of kappa_2 for each k so that power can be comparable across k (obtained by prior experimentation)
		avkappa2[i] = range(kappa2_lb, up, (up-kappa2_lb)/(ckappa2-1));
	}

	decl cresults = sizerc(vk)*ckap1mkap2*ckappa2;						// total # of experiments

	decl time = timer();

	decl mResults = zeros(cresults, 5);	 									  // stores results
	decl asNames = {"k", "kappa1-kappa2", "kappa2", "Conditional", "GKMC"};   // names of variables to store

	decl amquants = new array[sizerc(vk)];
	for(decl i = 0; i < sizerc(vk); ++i)
		amquants[i] = loadmat(sprint("CVtables\\k_",vk[i],"_level_",1-alpha,".in7"));   	   // array holding matrix of conditional quantile bins for the conditional test (x ~ cv)

	//Calculate power of conditional and GKMC AR tests
	parallel for(decl itask = 0; itask < cresults; ++itask)
	{
		ranseed(11111111); 													//keep same seed as in powerbounds.ox
		decl ik = idiv(itask,ckap1mkap2*ckappa2);
		decl ikap1mkap2 = idiv(itask - ckappa2*ik*ckap1mkap2,ckappa2);	//convert to index for kappa2, kap1mkap2 and k
		decl kap1mkap2 = vkap1mkap2[ikap1mkap2];
		decl k = vk[ik];
		decl ikappa2 = itask - ckappa2*(ikap1mkap2+ckap1mkap2*ik);		
		decl kappa2 = avkappa2[ik][ikappa2];				//value of kappa2 for DGP r under H1
	
		//Draw values under the alternative to calculate power:
		decl kappa1 = kap1mkap2+kappa2;				//largest eigenvalue
		decl mkappahat = new matrix[crepalt][2];
		decl dchi2cv = quanchi(1-alpha,k-1);
	
		for(decl i=0; i < crepalt; ++i)
		{
			decl mu = rann(k,2);
			decl mnc = zeros(k,2);				// initialize noncentrality (mean) of normals
			mnc[0][0] = sqrt(kappa1);			
			mnc[1][1] = sqrt(kappa2);			
			decl mX = mu+mnc;					// draw from non-central Wishart (via Normals)
			decl kaphat;
			eigensym(mX'mX, &kaphat);			// obtain its eigenvalues
			mkappahat[i][] = kaphat;	  	 		//store drawn eigenvalues
		}							  
		decl quants = amquants[ik];				// load conditional cv table (x ~ cv)
		decl mtestfunc = new matrix[crepalt][1];	// value of conditional test function for each draw (crepalt-vector of 0s and 1s)
		for(decl ix = 0; ix<crepalt; ++ix){
			decl x = mkappahat[ix][];				// largest~smallest eigenvalues
			decl idx = sumc(quants[][0].<x[0]);		// index of ceiling in x1 values (to be used as conditional cv)
			decl x0, x_lo, x_up;
			x0 = x[0];
			x_lo = quants[idx-1][0];
			x_up = quants[idx][0];
			decl y_up =  quants[idx][1], y_lo = quants[idx-1][1];
			decl dcv = interpolate(x0, x_up, x_lo, y_up, y_lo);			   // conditional critical value interpolated from table
			mtestfunc[ix][0] = x[1] .> dcv;			// conditional test function
		}				
		decl dpowerCond = meanc(mtestfunc);					   
		decl dpowerGKMC = meanc(mkappahat[][1].>dchi2cv);

		mResults[itask][] = k~kap1mkap2~kappa2~dpowerCond~dpowerGKMC;
	}
	savemat("PowerCondTest.in7",mResults,asNames);
	println("lapsed time: ", timespan(time));
}