/**	This file calculates the EMW (ALFD) and AMS upper bounds reported in GKM "A more powerfull subvector Anderson Rubin test in linear IV regression"
	It requires file DensityWishartEigvals.dll, and must be run on 64 bit Windows using OxMetrics version 7 or 7.1.

	NOTE: This code takes about 8 hours per case k to run on a 2.2GHz 44-core multi-threaded Windows server (with 88 virtual processors).
	It is quite memory-intensive.
**/

#include <oxstd.oxh>
#include <oxprob.oxh>
#include <oxfloat.oxh>
#import <database>

extern "DensityWishartEigvals,mhg_manyvals" density(const MAX, const alpha, const b, const X, const Y);
//extern "density_Wishart_Eigvals,mhg_manyvals" density(const MAX, const alpha, const b, const X, const Y);
// This file contains the function mhg_manyvals, which uses the algorithm (mhg) of Koev and Edelman (2005) in C for computing the 0F1 function of two matrix arguments
// the function multiplies 0F1 by exp(-sum(kappa+kappahat)/2), giving the joint density of the two eigenvalues up to an integrating constant


likelihood(const M, const k, const kappa, const kaphat)
/** Computes the value of the likelihood at kappa,kaphat
	in:	M integer, number of terms in zonal polynomial (truncation parameter)
		k degrees of freedom of Wishart
		kappa vector of eigenvalues of NC matrix
		kaphat vector of eigenvalues of Wishart matrix
	returns likelihood
**/
{
	return density(M,/*alpha*/2, k/2, kappa/4, kaphat);
}

DrawWishartEvals(const k, const kappa1, const kappa2)
/**	k: df, kappa1, kappa2: largest,smallest eigenvalue of NC matrix
	returns 1 x 2 vector kap1hat ~ kap2hat
**/
{
	decl mu = rann(k,2);
	decl mnc = zeros(k,2);				// initialize noncentrality (mean) of normals
	mnc[0][0] = sqrt(kappa1);			
	mnc[1][1] = sqrt(kappa2);			
	decl mX = mu+mnc;					// draw from non-central Wishart (via Normals)
	decl kaphat;
	eigensym(mX'mX, &kaphat);			// obtain its eigenvalues
	return kaphat;
}

main()									
{
	//parameters for H0
  	decl crep = 1e4;					// # of MC replications under null DGP, N_0 in the paper
	decl alpha = 0.05;					// significance level of test
	decl vk = <2,5,10,20>;				// vector containing different # of instruments
	decl ck = sizerc(vk);		
	decl ckappa1 = 42;									// # of points of kappa_1 under H0, N_{\kappa_1} in the paper
	decl kap1_lb = 0, kap1_ub = log(101);				// nuisance parameter range under H0, lower bound, uppper bound
	decl stepsize = (kap1_ub-kap1_lb)/(ckappa1-1);
	decl vkappa1 = exp(range(kap1_lb, kap1_ub, stepsize))-1;		  //Set grid for kappa_1 under H0.	   

	decl M = 200;						// truncation parameter for mhg

	//parameters for alternative
	decl crepalt = 1e5;	   					// # of MC repetitions under alternative, N_1 in the paper
	decl ckappa2 = 30; 						// # of kappa_2 points under H1
	decl kappa2_lb = 0.1;					// lower bound of kappa_2, very close to null
	decl vkap1mkap2 = 0~(2 .^<0:6>);		// kappa_1-kappa_2 points under H1 
	decl ckap1mkap2 = sizerc(vkap1mkap2);
	decl cH1 = ckappa2*ckap1mkap2;			//# of total points under H1, N_r in the paper
	decl avkappa2 = new array[ck];
	for(decl i = 0; i < ck; ++i){
		decl up = i == 0 ? 25 : i == 1 ? 30 : i == 2 ? 38 : 46;	 		// different range of kappa_2 for each k so that power can be comparable across k (obtained by prior experimentation)
		avkappa2[i] = range(kappa2_lb, up, (up-kappa2_lb)/(ckappa2-1));
	}

	println("Computing power bounds");
	println("# of MC reps under H0: ", crep, "; under H1: ", crepalt);
	println("# of DGPs under H0: ", ckappa1, "; under H1: ", cH1);

	decl mPower = <>;	 						//matrix that holds results to be stored in sfile 
	decl asPower = {"k", "kappa1-kappa2", "kappa2", "AMS", "EMW", "ALFD", "sizeAMS", "sizeEMW"};		//variable names in sfile
	decl sfile = "Powerbounds.in7";				//file name for storing results	

	for(decl ik = 0; ik < ck; ++ik){
		decl time = timer();							
		decl time1 = timer();
		decl k = vk[ik];
		println("\nDegrees of freedom: ", k);
		decl mResults = new matrix[cH1][8];	// matrix of results to store in mPower, for columns, see asPower
	
		//////////////////////////////////////////////////////
		//Draw eigenvalues under H0
		println("Drawing eigenvalues under H0..");
		decl amkappahat = new array[ckappa1];			// array [ckappa1] of crep x 2 matrices holding eigenvalues of Wishart draws
		parallel for(decl j = 0; j < ckappa1; ++j)
		{
			decl mkaphat = zeros(crep, 2);				//To store drawn eigenvalues.
			decl itask = ik*ckappa1 + j + 1;			
			ranseed(111111*(1+itask));					//set random seed (must be different across j for importance sampling)	
			for(decl i=0; i < crep; i++)
				mkaphat[i][] = DrawWishartEvals(k, vkappa1[j], 0);
			amkappahat[j] = mkaphat;
		}
		println("Done. Lapsed time: ", timespan(time1));
		time1 = timer();								// reset timer
		
		//////////////////////////////////////////////////////
		//Evaluate densities under H0 at draws from H0
		println("Evaluating likelihood under H0 across all DGPs..");
		decl amf = new array[ckappa1];				// array ckappa1 of crep x ckappa1 matrices holding densities
		decl mfbar = zeros(crep,ckappa1);			// crep x ckappa1 matrix holding \sum_j=1^ckappa1 f_j(y)/ckappa1, when y is drawn from the ckappa1 different DGPs under H_0
		parallel for(decl j = 0; j < ckappa1; ++j)
		{
			decl mlik = constant(.NaN, crep, ckappa1);  // matrix holding values of the density at drawn eigenvalues
		
			decl simult = 50; 							//number of points to calculate simultaneously in the likelihood.
			decl repsreq = crep/simult;
			for(decl i=0; i<repsreq; ++i)
			{
				decl kaphat = amkappahat[j][i*simult:(i+1)*simult-1][];
				for(decl j=0; j<ckappa1; ++j)
					mlik[i*simult:(i+1)*simult-1][j] = likelihood(M, k, vkappa1[j]~0, kaphat);		
			}
			amf[j] = mlik;
			mfbar[][j] = meanr(amf[j]);					//mean over all DGPs
		}
		decl amISweights = amf;							// initialize dimensions for array holding density weights
		for(decl j=0; j < ckappa1; ++j)
			for(decl jj = 0; jj < ckappa1; ++jj)
				amISweights[jj][][j] = amf[j][][jj];
		for(decl j = 0; j < ckappa1; ++j)
			amISweights[j] = amISweights[j]./mfbar;
		println("Done. Lapsed time: ", timespan(time1));
		time1 = timer();							// reset timer
	
		//////////////////////////////////////////////////////
		//Evaluate densities under H1 at draws from H0
		println("Evaluating likelihood under H1 across all DGPs..");
		decl amg = new array[cH1];							// array [cH1] of crep x ckappa1 matrices holding densities under H1
		parallel for(decl r = 0; r < cH1; ++r)
		{
			decl ikap1mkap2 = idiv(r,ckappa2);				//index of kap1mkap2
			decl kap1mkap2 = vkap1mkap2[ikap1mkap2];		//value of kap1mkap2 for DGP r under H1
			decl ikappa2 = r - ckappa2*ikap1mkap2;			//index of kappa2
			decl kappa2 = avkappa2[ik][ikappa2];			//value of kappa2 for DGP r under H1
		
			decl mg = new matrix[crep][ckappa1];			//values of g(y) under H_1, when y is drawn from the N_{\kappa_1} different DGPs under H_0
			decl simult = 50;								//speeds up evaluation (but if you choose larger value it gets slower)
			decl repsreq = crep/simult;
			for(decl i=0; i < repsreq; i++)
			{
				for(decl j=0; j < ckappa1; j++)
				{
					decl kaphat = amkappahat[j][i*simult:(i+1)*simult-1][];																  
					mg[i*simult:(i+1)*simult-1][j] = likelihood(M, k, (kappa2+kap1mkap2)|kappa2, kaphat);		// likelihood under alternative
				}
			}
			amg[r] = mg;	
		}
		println("Done. Lapsed time: ", timespan(time1));
		time1 = timer();							// reset timer

		///////////////////////////////////////////////////////////////////////
		//Compute AMS one-point least favorable distribution and corresponding
		//cv (up to step 7 of AMS algorithm in section S.2.3.1)
		println("Computing AMS one-point least favorable distribution and cv..");		
		decl vlstar  = new matrix[ckap1mkap2*ckappa2][1]; 			//index of one-point least favorable distribution
		decl vcvAMS  = new matrix[ckap1mkap2*ckappa2][1]; 			//stores cv of one-point least favorable distribution
		decl tol = 1e-4;											//desired degree of accuracy in step 5 of the algorithm (computation of cvs)
		parallel for(decl r = 0; r < cH1; ++r)
		{
			decl vvarkappa0 = zeros(ckappa1, 1);					//vector holding critical values (denoted \varkappa_j in the paper) for each NP test at alternative j=1,...,ckappa1
			decl mRP = zeros(ckappa1, ckappa1); 					//Null rejection probabilities.
			decl distance;											//distance of rej prob from alpha
			decl adj;												//adjustment factor for the increments in the algorithm that finds the cvs
		
			//Step 5: find critical value so that NP test has level alpha at DGP s
			for(decl s=0; s<ckappa1; ++s)
			{
				adj = 2;
				//Obtain starting value \varkappa^0:
				decl mLR = amg[r]./amISweights[s]./mfbar;
				vvarkappa0[s] = quantilec(mLR[][s], 1-alpha);
		
				distance = meanr(meanc((mLR .> vvarkappa0[s]).*amISweights[s]))-alpha;	 // compute deviation from level alpha		
				// and adjust cv to hit level alpha at each DGP
				for(decl i = 0; i < 100; ++i){
					if(fabs(distance) < tol)
						break;
					vvarkappa0[s] += vvarkappa0[s]*adj*distance;  					// adjust cv
					decl distance1 = meanr(meanc((mLR .> vvarkappa0[s]).*amISweights[s]))-alpha;	 		
					adj = distance*distance1 < 0 ? adj/2 : adj*1.03; 	 			//if switches sign, decrease adjustment, else increase it slightly.				   
					distance = distance1;
				}
			}
		
			//Step 6 (a): compute NRP of each test under every DGP
			for(decl t=0; t<ckappa1; t++)
				for(decl s=0; s<ckappa1; s++)
					mRP[s][t] = meanr(meanc(((amg[r]./amISweights[s]./mfbar) .> vvarkappa0[s]).*amISweights[t]));
			//Step 6 (b): compute distance of size of each test from alpha
			decl vD = maxr(mRP-alpha);
			//Step 7: find test with size closest to alpha
			decl lstar = mincindex(vD);			   			
			mResults[r][6] = maxr(mRP[lstar][]);			//store size of test at l*
			vcvAMS[r] = vvarkappa0[lstar];
			vlstar[r] = lstar;
		}
		println("Done. Lapsed time: ", timespan(time1));
		time1 = timer();							// reset timer

		///////////////////////////////////////////////////////////////////////
		//Compute ALFD (up to step 8 of EMW algorithm in section S.2.3.2)
		println("Computing ALFD weights and cvs..");
		decl mALFDweights  = new matrix[cH1][ckappa1];				//holds ALFD weights \lambda_j (step 5)
		decl vcvEMW 	   = new matrix[cH1][1];					//holds cv \varkappa* (step 6)
		decl vcvALFD 	   = new matrix[cH1][1];					//cv \varkappa (step 8)
		parallel for(decl r = 0; r < cH1; ++r)
		{
			// Find candidate Least Favorable distribution Lambda*
			decl vmu = constant(-2, 1, ckappa1);					// initialize weights of ALFD (step 4 of algorithm)
			decl cO = 600;											// # of iterations for ALFD
			decl omega = 2;											// slope of update
			decl vRP = zeros(ckappa1,1);							// rejection frequencies
			decl mtest = new matrix[crep][ckappa1];					// holds test function 1_{g(y)>\Sum_j=1^ckappa1 exp{mu_j}*f_j}
			for(decl i = 0; i < cO; ++i){
				decl weights = exp(vec(vmu));
				for(decl j = 0; j < ckappa1; ++j)
					mtest[][j] = amg[r][][j] .> amf[j]*weights;
				for(decl j = 0; j < ckappa1; ++j)
					vRP[j] = meanc(meanr(amISweights[j].*mtest));
						
				vmu += omega.*(vRP'-alpha);
			}
			
			decl vALFDweights = exp(vmu)./sumr(exp(vmu));			// compute ALFD weights \lambda*_j (step 5 of the algorithm)
			mALFDweights[r][] = vALFDweights;						// store for later
			// STEP 6: find critical value for Lambda*
			decl vquants = zeros(ckappa1,1);						// 1-alpha quantiles of LR test at various DGPs
			// estimate the various quantiles without Importance Sampling in order to set a grid of cvs
			decl mLR = new matrix[crep][ckappa1];					// to store LR statistic under ALFD weights
			for(decl j = 0; j < ckappa1; ++j)		 				// j indexes true DGP
				mLR[][j] = amg[r][][j]./(amf[j]*vALFDweights');		// likelihood ratio for H0=ALFD at true DGP j
			vquants = quantilec(mLR, 1-alpha);						// 1-alpha quantiles for each distribution j
		
			decl dcv = max(vquants);								// initialize \varkappa* (step 6) at highest quantile
			
			decl adj = 2;
			decl cgridcv = 120;										// maximum number of iterations
			mtest = mLR .> dcv;
			for(decl t=0; t<ckappa1; t++)
				vRP[t] = meanr(meanc(mtest.*amISweights[t]));		// compute RP with IS
			decl distance = vALFDweights*(vRP-alpha);				// RP of ALFD LR test at DGP given by ALFD
			for(decl i = 0; i < cgridcv; ++i){
				if(fabs(distance) < tol)
					break;
				dcv += dcv*adj*distance;  							//if it overrejects, increase cv, else decrease
				mtest = (mLR .> dcv);
				for(decl t=0; t<ckappa1; t++)
					vRP[t] = meanr(meanc(mtest.*amISweights[t]));
				decl distance1 = vALFDweights*(vRP-alpha);	 		
				adj = distance*distance1 < 0 ? adj/2 : adj*1.03; 	//if switches sign, decrease adjustment, else increase it slightly.
				distance = distance1;
			}
			vcvEMW[r] = dcv; 										// store cv \varkappa* (step 6)
			mResults[r][7] = max(vRP);								// store size of test with \Lambda* weights

			// STEP 8: find cv st test has size alpha 
			adj = 2;
			mtest = mLR .> dcv;
			for(decl t=0; t<ckappa1; ++t)
				vRP[t] = meanr(meanc(mtest.*amISweights[t]));
			distance = max(vRP-alpha);
			for(decl i = 0; i < cgridcv; ++i){
				if(fabs(distance) < tol)
					break;
				dcv += dcv*adj*distance;  							 //if we overreject, increase cv, else decrease
				mtest = (mLR .> dcv);
				for(decl t=0; t<ckappa1; ++t)
					vRP[t] = meanr(meanc(mtest.*amISweights[t]));
				decl distance1 = max(vRP-alpha);	 		
				adj = distance*distance1 < 0 ? adj/2 : adj*1.03; 	 //if switches sign, decrease adjustment, else increase it slightly.				   
				distance = distance1;
			}
			vcvALFD[r] = dcv; 										 //store cv \varkappa (step 8)
		}
		println("Done. Lapsed time: ", timespan(time1));
 		time1 = timer();											// reset timer

		delete amISweights;											// clear memory
		delete amg;
		delete mfbar;		  
		delete amf;

		///////////////////////////////////////////////////////////////////////
		//Compute AMS and EMW power bounds -- simulate under H1
		println("Computing AMS and EMW bounds..");
		parallel for(decl r = 0; r < cH1; ++r)
		{
			ranseed(11111111); 		
			decl ikap1mkap2 = idiv(r,ckappa2);					//index of kappa1-kappa2 under H1
			decl kap1mkap2 = vkap1mkap2[ikap1mkap2];			//value of kappa1-kappa2 for DGP r under H1
			decl ikappa2 = r - ckappa2*ikap1mkap2;				//index of kappa2 under H1
			decl kappa2 = avkappa2[ik][ikappa2];				//value of kappa2 for DGP r under H1
																					
			//Draw values under the alternative to calculate power:
			decl mLR_lstar = zeros(crepalt, 1);			// to store value of LR for AMS test at one-point LF distn indexed by lstar
			decl mg_H1 = new matrix[crepalt][1];		// simulated values of lik g(y) under H_1, when y is drawn from g (H_1)
			decl mf_ALFD = zeros(crepalt, 1);		 	// lik under H0 with weights Lambda* (one-point for AMS, ALFD for EMW)
		
			decl simult = 50;
			decl repsreq = crepalt/simult;
		
			decl kappa1 = kap1mkap2+kappa2;				//largest eigenvalue
			decl mkappahat = new matrix[crepalt][2];	//draws of eigenvalues of nc Wishart
		
			for(decl i = 0; i < crepalt; ++i)
				mkappahat[i][] = DrawWishartEvals(k, kappa1, kappa2);	  	 
		
			decl fH0 = zeros(crepalt, ckappa1);
		
			for(decl i=0; i < repsreq; i++)
			{
				//Calculate Likelihoods:
				decl kaphattmp = zeros(simult, ckappa1);
				mg_H1[i*simult:(i+1)*simult-1] = likelihood(M, k,  kappa1|kappa2, mkappahat[i*simult:(i+1)*simult-1][]);
				for(decl j = 0; j < ckappa1; ++j)
					kaphattmp[][j] = likelihood(M, k, vkappa1[j]|0, mkappahat[i*simult:(i+1)*simult-1][]); 
		
				fH0[i*simult:(i+1)*simult-1][] = kaphattmp;
				mf_ALFD[i*simult:(i+1)*simult-1] = kaphattmp*mALFDweights[r][]';		// \lambda_j*f_j(y)	
				mLR_lstar[i*simult:(i+1)*simult-1] = mg_H1[i*simult:(i+1)*simult-1]./kaphattmp[][vlstar[r]];		//AMS NP statistic, evaluated at one-point LF distn indexed by lstar
			}

			mResults[r][:5] = k~kap1mkap2~kappa2
						  ~ meanc(mLR_lstar .> vcvAMS[r])
						  ~ meanc(mg_H1 .> vcvEMW[r]*mf_ALFD)
						  ~ meanc(mg_H1 .> vcvALFD[r]*mf_ALFD);
		
		}
		println("Done. Lapsed time: ", timespan(time1));
		mPower |= mResults;
		savemat(sfile,mPower,asPower);
		println("Done with k = ", k, ". Lapsed time: ", timespan(time),"\n");
	}
}